package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kvector.api.IStruct2i

// for getting tiles directly, avoiding manual layer specification
interface ITileAccess : ICellAccess {
	// relative
	fun getTile(x: Int, y: Int): ITileState
	fun getTile(pos: IStruct2i) = getTile(pos.component1(), pos.component2())
}

sealed class TileView(parent: ICellAccess) : ITileAccess, ICellAccess by parent {
	private class Foreground(parent: ICellAccess) : TileView(parent) {
		override fun getTile(x: Int, y: Int): ITileState {
			return getCell(x, y).foreground
		}
	}

	private class Background(parent: ICellAccess) : TileView(parent) {
		override fun getTile(x: Int, y: Int): ITileState {
			return getCell(x, y).background
		}
	}

	companion object {
		fun foreground(parent: ICellAccess, xOffset: Int = 0, yOffset: Int = 0): TileView {
			if (xOffset == 0 && yOffset == 0) {
				return Foreground(parent)
			} else {
				return Foreground(OffsetCellAccess(parent, xOffset, yOffset))
			}
		}

		fun foreground(parent: ICellAccess, offset: IStruct2i): TileView {
			return Foreground(OffsetCellAccess(parent, offset))
		}

		fun foreground(parent: ICellAccess, offset: ChunkPos): TileView {
			return Foreground(OffsetCellAccess(parent, offset))
		}

		fun background(parent: ICellAccess, xOffset: Int = 0, yOffset: Int = 0): TileView {
			if (xOffset == 0 && yOffset == 0) {
				return Background(parent)
			} else {
				return Background(OffsetCellAccess(parent, xOffset, yOffset))
			}
		}

		fun background(parent: ICellAccess, offset: IStruct2i): TileView {
			return Background(OffsetCellAccess(parent, offset))
		}

		fun background(parent: ICellAccess, offset: ChunkPos): TileView {
			return Background(OffsetCellAccess(parent, offset))
		}
	}
}
